
//WndTimer.h : header file

#ifndef __WNDTIMER_H__
#define __WNDTIMER_H__

#pragma warning(disable:4786)

#include <deque>
#include <map>

using namespace std;

//Windows Timer Class
class CWndTimer
{
  typedef void (*PTIMERPROC)(void*);
public:
  //CONSTRUCTOR
  CWndTimer();
  //DESTRUCTOR
  virtual ~CWndTimer();
  //Creation Functions
  int Create(HWND const& rhWnd, UINT uiID, UINT uiPeriod);
  bool Create(PTIMERPROC pTimerProc, LPVOID pArg, UINT uiPeriod);
  //Starting
  bool Start();
  //Stopping
  bool Stop();
  //Delete the Timers of the given Window
  static bool Delete(HWND const& rhWnd);
  //Check if is active
  bool IsRunning();
  //ID Getter
  UINT GetID();
  //Period Setter
  void SetPeriod(UINT uiPeriod);
  //Period Getter
  UINT GetPeriod();
  //Thread Function is a friend
  friend UINT TimerThreadProc(LPVOID pParam);
  //Timer Message
  static const UINT UWM_TIMER;

private:
  //Disallow copy
  CWndTimer(const CWndTimer&); 
  CWndTimer& operator=(const CWndTimer&);
  //Critical Section for protecting the access to the static members
  static CRITICAL_SECTION sm_CS;
  static bool sm_bIniCS;
  //Data Map
  static multimap<HWND, CWndTimer*> sm_oMultiMap;
  //Data Queue
  static deque<CWndTimer*> sm_oQueue;
  //Thread Handle
  static HANDLE  sm_hThread;
  //Event Handle
  static HANDLE sm_hEvent;
  //Thread Run Flag
  static bool sm_bThreadRunning;
  //Timer Types
  enum { TYPE_PROC=0, TYPE_WIN=1 };
  int m_iType;
  //Timer ID
  UINT m_uiID;
  //Timer Period
  UINT m_uiPeriod;
  //Timer Procedure
  //If this Timer Procedure is called from other threads too, the accessed data should
  //be protected using appropriate synchronization objects
  PTIMERPROC m_pTimerProc;
  //The Argument to Timer Procedure
  LPVOID m_pArg;
  //Scheduled Ticks's Tick Count
  DWORD m_uiTickNext;
  //Creation Flag
  bool m_bCreated;
  //Run Flag
  bool m_bRunning;
};

//ID Getter
inline UINT CWndTimer::GetID()
{
  return m_uiID;
}

//Check if is active
inline bool CWndTimer::IsRunning()
{
  return m_bRunning;
}

//Period Getter
inline UINT CWndTimer::GetPeriod()
{
  return m_uiPeriod;
}

#endif // __WNDTIMER_H__
